#include <Arduino.h>
#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEClient.h>

#define SERVICE_UUID        "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID "beb5483e-36e1-4688-b7f5-ea07361b26a8"

BLEClient* pClient = NULL;
bool connected = false;

class MyClientCallbacks : public BLEClientCallbacks {
    void onConnect(BLEClient* pClient) {
        connected = true;
        Serial.println("Connected to server");
    }

    void onDisconnect(BLEClient* pClient) {
        connected = false;
        Serial.println("Disconnected from server");
    }
};

void setup() {
    Serial.begin(115200);
    delay(1000);

    BLEDevice::init("Client");
    pClient = BLEDevice::createClient();
    pClient->setClientCallbacks(new MyClientCallbacks());

    BLEAddress serverAddress("64:e8:33:89:fe:0e"); // Replace with server's Bluetooth address
    if (pClient->connect(serverAddress)) { 
        Serial.println("Connecting...");
    } else {
        Serial.println("Failed to connect to server");
    }
}

void loop() {
    if (connected) {
        BLERemoteService* pRemoteService = pClient->getService(SERVICE_UUID);
        if (pRemoteService != nullptr) {
            BLERemoteCharacteristic* pCharacteristic = pRemoteService->getCharacteristic(CHARACTERISTIC_UUID);
            if (pCharacteristic != nullptr) {
                String value = pCharacteristic->readValue(); // Get the value as Arduino String
                Serial.println(value); // Output the information received from the server
            } else {
                Serial.println("Failed to find characteristic");
            }
        } else {
            Serial.println("Failed to find service");
        }
        delay(1000); // Add delay to avoid continuous reading
    }
}
