#include <Arduino.h>
#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEServer.h>

#define SERVICE_UUID        "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID "beb5483e-36e1-4688-b7f5-ea07361b26a8"

BLEServer* pServer = NULL;
BLECharacteristic* pCharacteristic = NULL;
bool deviceConnected = false;
bool oldDeviceConnected = false;

class MyServerCallbacks : public BLEServerCallbacks {
    void onConnect(BLEServer* pServer) {
      deviceConnected = true;
    };

    void onDisconnect(BLEServer* pServer) {
      deviceConnected = false;
    }
};

void setup() {
    Serial.begin(115200);
    delay(1000);

    BLEDevice::init("Server");
    pServer = BLEDevice::createServer();
    pServer->setCallbacks(new MyServerCallbacks());

    BLEService* pService = pServer->createService(SERVICE_UUID);
    pCharacteristic = pService->createCharacteristic(
                                         CHARACTERISTIC_UUID,
                                         BLECharacteristic::PROPERTY_READ |
                                         BLECharacteristic::PROPERTY_WRITE |
                                         BLECharacteristic::PROPERTY_NOTIFY
                                       );

    pService->start();
    pServer->getAdvertising()->start();
}

void loop() {
    // Check if device is connected
    if (deviceConnected) {
        for (int i = 1; i <= 10; i++) {
            pCharacteristic->setValue(String(i).c_str());
            pCharacteristic->notify();
            Serial.println("Sent: " + String(i));
            delay(1000); // 1 second delay between messages
        }
        
        // Send the final message
        pCharacteristic->setValue("All messages sent.");
        pCharacteristic->notify();
        Serial.println("Sent: All messages sent.");

        // Wait until device disconnects
        while (deviceConnected) {
            delay(100);
        }
    }
    
    // Disconnect handling
    if (!deviceConnected && oldDeviceConnected) {
        delay(500); // give the bluetooth stack the chance to get things ready
        pServer->startAdvertising(); // restart advertising
        Serial.println("start advertising");
        oldDeviceConnected = deviceConnected;
    }
    
    // Connect handling
    if (deviceConnected && !oldDeviceConnected) {
        oldDeviceConnected = deviceConnected;
    }
}
